# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.1.beta2'
date = '2023-06-03'
banner = 'SageMath version 10.1.beta2, Release Date: 2023-06-03'
